//
// CocosCreator 热更打包模块
//
var fs = require("fs");
var FsExtra = require("fs-extra");
var crypto = require("crypto");
var path = require("path");
var AssetsDB = Editor.assetdb;
var FsUtils = require("./FsUtils");

module.exports = {

    /**
     * 编译子包清单
     * @param {string} targetDir 
     * @param {Package} packInfo 
     * @param {boolean} isDebug 
     */
    getAssetsDB() {
        if (Editor.isMainProcess) {
            return Editor.assetdb;
        }
        return Editor.remote.assetdb;
    },
    readDirToUUID(root, dir, subPack, mainPackNoGen, buildType, destDir) {
        if (root === null)
            dir = path.join(Editor.Project.path, dir);
        var stat = fs.statSync(dir);
        if (!stat.isDirectory()) {
            return;
        }
        let subpaths = fs.readdirSync(dir),
            subpath, size, md5, compressed, relative, extname;
        for (let i = 0; i < subpaths.length; ++i) {
            if (subpaths[i][0] === '.') continue;
            subpath = path.join(dir, subpaths[i]);
            stat = fs.statSync(subpath);
            if (stat.isDirectory()) {
                this.readDirToUUID(subpath, subpath, subPack, mainPackNoGen, buildType, destDir);
            } else if (stat.isFile()) {
                // Size in Bytes
                size = stat['size'];
                md5 = crypto.createHash('md5').update(fs.readFileSync(subpath, 'binary')).digest('hex');
                relative = path.relative(Editor.Project.path, subpath);

                relative = relative.replace(/\\/g, '/');
                compressed = path.extname(subpath).toLowerCase() === '.meta';
                if (!compressed) {
                    Editor.assetdb.queryAssets("db://" + relative, ['native-asset', 'animation-clip', 'audio-clip', 'bitmap-font', 'javascript', 'json', 'prefab', 'scene',
                        'texture-packer', 'sprite-frame', 'texture', 'text', 'label-atlas', 'raw-asset', 'font', 'spine', 'auto-atlas', 'sprite-atlas', 'dragonbones-atlas'
                    ], function (err, results) {
                        results.forEach(function (result, index) {
                            extname = path.extname(result.path).toLowerCase();
                            var extNameRaw = ['.png',
                                '.mp3',
                                '.atlas',
                                '.jpg',
                                '.ogg',
                                '.wav',
                            ]

                            if (extNameRaw.includes(extname)) {
                                pathUUID = "res/raw-assets/" + result.uuid.charAt(0) + result.uuid.charAt(1) + "/" + result.uuid + extname;
                                if (fs.existsSync(Editor.Project.path + "/build/" + buildType + "/" + pathUUID)) {
                                    mainPackNoGen.push(pathUUID);
                                    subPack.assets[pathUUID] = {
                                        'md5': md5,
                                        'size': size,
                                    }
                                    FsExtra.copy(Editor.Project.path + "/build/" + buildType + "/" + pathUUID,
                                        destDir + pathUUID
                                        , function (err) {
                                        })
                                }
                                pathUUIDImport = "res/import/" + result.uuid.charAt(0) + result.uuid.charAt(1) + "/" + result.uuid + ".json";
                                if (fs.existsSync(Editor.Project.path + "/build/" + buildType + "/" + pathUUIDImport)) {
                                    var statImport = fs.statSync(Editor.Project.path + "/build/" + buildType + "/" + pathUUIDImport);
                                    var md5Import = crypto.createHash('md5').update(fs.readFileSync(Editor.Project.path + "/build/" + buildType + "/" + pathUUIDImport, 'binary')).digest('hex');
                                    mainPackNoGen.push(pathUUIDImport);
                                    subPack.assets[pathUUIDImport] = {
                                        'md5': md5Import,
                                        'size': statImport['size'],
                                    }
                                    FsExtra.copy(Editor.Project.path + "/build/" + buildType + "/" + pathUUIDImport,
                                        destDir + pathUUIDImport
                                        , function (err) {
                                        })
                                }
                            } else if (extname == ".fnt" || extname == ".prefab" || extname == ".json" || extname == ".fire") {
                                pathUUIDImport = "res/import/" + result.uuid.charAt(0) + result.uuid.charAt(1) + "/" + result.uuid + ".json";

                                if (fs.existsSync(Editor.Project.path + "/build/" + buildType + "/" + pathUUIDImport)) {
                                    var statImport = fs.statSync(Editor.Project.path + "/build/" + buildType + "/" + pathUUIDImport);
                                    var md5Import = crypto.createHash('md5').update(fs.readFileSync(Editor.Project.path + "/build/" + buildType + "/" + pathUUIDImport, 'binary')).digest('hex');
                                    mainPackNoGen.push(pathUUIDImport);
                                    subPack.assets[pathUUIDImport] = {
                                        'md5': md5Import,
                                        'size': statImport['size'],
                                    }
                                    FsExtra.copy(Editor.Project.path + "/build/" + buildType + "/" + pathUUIDImport,
                                        destDir + pathUUIDImport
                                        , function (err) {
                                        })
                                }
                            }
                        });
                    });
                }
            }
        }
    },
    readDirManifest(base, dir, obj, noInclude, destDir, packName) {
        var stat = fs.statSync(dir);
        if (!stat.isDirectory()) {
            return;
        }
        var subpaths = fs.readdirSync(dir),
            subpath, size, md5, relative;
        for (var i = 0; i < subpaths.length; ++i) {
            if (subpaths[i][0] === '.') {
                continue;
            }
            subpath = path.join(dir, subpaths[i]);
            stat = fs.statSync(subpath);
            if (stat.isDirectory()) {
                this.readDirManifest(base, subpath, obj, noInclude, destDir, packName);
            } else if (stat.isFile()) {
                // Size in Bytes
                size = stat['size'];
                md5 = crypto.createHash('md5').update(fs.readFileSync(subpath)).digest('hex');
                relative = path.relative(base, subpath);
                relative = relative.replace(/\\/g, '/');
                relative = encodeURI(relative);
                if (!noInclude.includes(relative)) {
                    obj[relative] = {
                        'size': size,
                        'md5': md5
                    };
                }  
            }
        }
    },
    generateManifest(url, subpack) {
        var manifest = Object.create(null);
        let packageUrl = url.trim() + "/" + subpack.name;
        packageUrl = packageUrl.replace(/\/$/, "");
        let remoteVersionUrl = packageUrl + "/version.manifest";
        let remoteManifestUrl = packageUrl + "/project.manifest";

        manifest.version = subpack.version;
        manifest.name = subpack.name;
        manifest.packageUrl = packageUrl;
        manifest.remoteVersionUrl = remoteVersionUrl;
        manifest.remoteManifestUrl = remoteManifestUrl;
        manifest.searchPaths = [];
        manifest.assets = subpack.assets || Object.create(null);
        return manifest;
    },
    writeManifest(destDir, manifest) {
        destDir = path.join(Editor.Project.path, destDir + "/" + manifest.name + "/");
        if (!fs.existsSync(destDir)) {
            FsExtra.mkdir(destDir, function (err) {
                let file = path.join(destDir, "/project.manifest");
                FsExtra.writeJSON(file, manifest, function (err) {
                    Editor.log("Ghi file " + manifest.name + " project.manifest" + " thành công");
                });
                let version = JSON.parse(JSON.stringify(manifest));
                delete version.assets;
                delete version.searchPaths;
                file = path.join(destDir, "/version.manifest");
                FsExtra.writeJSON(file, version, function (err) {
                    Editor.log("Ghi file " + manifest.name + " version.manifest" + " thành công");
                });
            });
        } else {
            let file = path.join(destDir, "/project.manifest");
            FsExtra.writeJSON(file, manifest, function (err) {
                Editor.log("Ghi file " + manifest.name + " project.manifest" + " thành công");
            });
            let version = JSON.parse(JSON.stringify(manifest));
            delete version.assets;
            delete version.searchPaths;
            file = path.join(destDir, "/version.manifest");
            FsExtra.writeJSON(file, version, function (err) {
                Editor.log("Ghi file " + manifest.name + " version.manifest" + " thành công");
            });
        }

    },


}